"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const core_1 = require("@angular-devkit/core");
const schematics_1 = require("@angular-devkit/schematics");
const glob_1 = __importDefault(require("glob"));
const typedoc_1 = require("typedoc");
const typescript_1 = require("typescript");
const tree_1 = require("../../utility/tree");
const workspace_1 = require("../../utility/workspace");
function parseFriendlyUrlFragment(value) {
    const friendly = value
        .toLowerCase()
        // Remove special characters.
        .replace(/[_~`@!#$%^&*()[\]{};:'/\\<>,.?=+|"]/g, '')
        // Replace space characters with a dash.
        .replace(/\s/g, '-')
        // Remove any double-dashes.
        .replace(/--/g, '-');
    return friendly;
}
/**
 * Returns anchor IDs to be used for same-page linking.
 */
function getAnchorIds(json) {
    var _a;
    const anchorIdMap = {};
    (_a = json.children) === null || _a === void 0 ? void 0 : _a.filter((child) => {
        var _a;
        const kindString = (_a = child.kindString) === null || _a === void 0 ? void 0 : _a.toLocaleUpperCase();
        return kindString && kindString !== 'VARIABLE';
    }).forEach((child) => {
        const kindString = parseFriendlyUrlFragment(child.kindString);
        const friendlyName = parseFriendlyUrlFragment(child.name);
        const anchorId = `${kindString}-${friendlyName}`;
        anchorIdMap[child.name] = anchorId;
    });
    return anchorIdMap;
}
function applyTypeDocDefinitions(documentationJson, project) {
    return (tree, context) => __awaiter(this, void 0, void 0, function* () {
        const publicApiPath = core_1.normalize(`${project.sourceRoot}/public-api.ts`);
        const publicApiContents = tree_1.readRequiredFile(tree, publicApiPath);
        context.logger.info(`Primary entry point: ${publicApiPath}`);
        const files = [publicApiPath];
        // Add entry points for any components and directives that are not explicitly listed in public-api.ts.
        // TODO: The following can be removed once we export our components/directives from the public-api.ts file.
        glob_1.default
            .sync(`${project.sourceRoot}/**/*.+(component|directive).ts`, {
            nodir: true,
            ignore: ['**/fixtures/**', '**/testing/**'],
        })
            .forEach((file) => {
            if (!publicApiContents.includes(file.replace(project.sourceRoot, '').replace(/.ts$/, ''))) {
                context.logger.warn(`Adding another entry point for "${file}" because it is not listed in the public-api.ts file. Should it be?`);
                files.push(core_1.normalize(file));
            }
        });
        const app = new typedoc_1.Application();
        app.bootstrap({
            exclude: ['**/node_modules/**', '**/fixtures/**', '**/*.spec.ts'],
            entryPoints: files,
            excludeExternals: true,
            excludeInternal: true,
            excludePrivate: true,
            excludeProtected: true,
            logLevel: 'Verbose',
        });
        app.options.setCompilerOptions(files, {
            esModuleInterop: true,
            experimentalDecorators: true,
            module: typescript_1.ModuleKind.ES2020,
            moduleResolution: typescript_1.ModuleResolutionKind.NodeJs,
            resolveJsonModule: true,
            target: typescript_1.ScriptTarget.ES2017,
            baseUrl: './',
        }, undefined);
        const typedocProject = app.convert();
        if (typedocProject) {
            const json = app.serializer.toObject(typedocProject);
            let processed;
            // TypeDoc creates multiple sections if there are multiple entry points.
            // The `@skyux/docs-tools` library expects the documentation.json definitions to be "flattened",
            // so we need to combine the multiple sections into one.
            // TODO: The following condition can be removed once we export our components/directives from the public-api.ts file.
            if (files.length > 1) {
                processed = {
                    children: [],
                };
                json.children.forEach((entrypoint) => {
                    if (entrypoint.children) {
                        processed.children.push(...entrypoint.children);
                    }
                });
            }
            else {
                processed = json;
            }
            const anchorIds = getAnchorIds(processed);
            documentationJson.anchorIds = anchorIds;
            documentationJson.typedoc = processed;
        }
        else {
            throw new schematics_1.SchematicsException('TypeDoc project generation failed. ' +
                'This usually occurs when the target TypeScript project cannot compile or is invalid. ' +
                'Try running `ng build` to debug any compiler issues.');
        }
    });
}
function getDocumentationJsonPath(tree, project) {
    const ngPackageJson = JSON.parse(tree_1.readRequiredFile(tree, `${project.root}/ng-package.json`));
    const outputPath = core_1.resolve(`${project.root}`, ngPackageJson.dest);
    return `${outputPath}/documentation.json`;
}
function ensureDocumentationJson(tree, documentationJsonPath) {
    /* istanbul ignore else */
    if (!tree.exists(documentationJsonPath)) {
        tree.create(documentationJsonPath, '{}');
    }
}
function applyCodeExamples(documentationJson, project) {
    return (tree, context) => {
        const codeExamples = [];
        tree
            .getDir(`${project.root}/documentation/code-examples`)
            .visit((filePath) => {
            context.logger.info(`Processing code example: ${filePath}`);
            codeExamples.push({
                fileName: core_1.basename(filePath),
                filePath,
                rawContents: tree_1.readRequiredFile(tree, filePath),
            });
        });
        documentationJson.codeExamples = codeExamples;
    };
}
function getDocumentationJson(tree, project) {
    const documentationJsonPath = getDocumentationJsonPath(tree, project);
    ensureDocumentationJson(tree, documentationJsonPath);
    return JSON.parse(tree_1.readRequiredFile(tree, documentationJsonPath));
}
function updateDocumentationJson(documentationJson, project) {
    return (tree) => {
        const documentationJsonPath = getDocumentationJsonPath(tree, project);
        tree.overwrite(documentationJsonPath, JSON.stringify(documentationJson, undefined, 2));
    };
}
function generateDocumentation(options) {
    return (tree, context) => __awaiter(this, void 0, void 0, function* () {
        const { workspace } = yield workspace_1.getWorkspace(tree);
        const { project, projectName } = yield workspace_1.getProject(workspace, options.project || workspace.extensions.defaultProject);
        context.logger.info(`Attempting to generate documentation for project "${projectName}"...`);
        if (project.extensions.projectType !== 'library') {
            throw new schematics_1.SchematicsException('Only library projects can generate documentation.');
        }
        const documentationJson = getDocumentationJson(tree, project);
        return schematics_1.chain([
            applyTypeDocDefinitions(documentationJson, project),
            applyCodeExamples(documentationJson, project),
            updateDocumentationJson(documentationJson, project),
        ]);
    });
}
exports.default = generateDocumentation;
//# sourceMappingURL=documentation.schematic.js.map